 // Initialization
  
    clear
    mode(0);
    ieee(1);
    clearglobal;
    
// directories & names

    pwd;
    
    T_wd = uigetdir(ans,"Go to main directory (contains \1_data, \2_results)");
    
    if T_wd == "" then
            chdir(T_wd);
            x_mdialog("Program ended","Caution: remove files from \2_results","Press ok");
            abort;
        end
        
    chdir(T_wd);
  
    T_data = T_wd + "\1_data";
    T_results = T_wd + "\2_results";
    
    chdir(T_data);
    
    name_  = uigetfile(["*.txt"],T_data,"Select data file");
    
    name = part(name_,length(T_data)+2:length(name_)-4);
    
    // constants
    
    i_fig = 1;
    xfs_title = 1.75;
    xfs_leg = 1.5;
    xfs_axis = 3;
    fs = 1;
    
    pfig = 5;
    
// run functions
    
      for i = 1:pfig
        close;
      end;
    
// parameters for filtering
    
    er_tol = 0.01;                // precision of the measure
    r_tol = 1.75;                 // security factor for tolerance : assumed (%%)
    r_lgth = 7.5;                 // ratio of length : assumed (%%)

// I.1 experimental data and names

// Data for the column

//// initialisation of variables

    entry = [];
    t_exp = [];
    y_exp = [];
    t_exp_r = [];
    delta_t = [];
    delta_y = [];
    a1 = [];
    a2 = [];
    length_cell = [];
    tcell = cell();
    ycell = cell();
    na1 = [];
    na2 = [];
    t_exp_rr = [];
    y_exp_rr = [];
    delta_y_exp_rr = [];
    zna_ = [];
    znb_ = [];
    zna = [];
    znb = [];
    t_exp_rrz = [];
    y_exp_rrz = [];
    t_select = [];
    y_select = [];
    t_exp_rc = [];
    y_exp_rc = [];
    t_staircase = [];
    y_staircase = [];
    
// experimental data
    
    entry = read(name + ".txt",-1,2);                                   //--> data file
    
    t_exp = (entry(:,1)-entry(1,1));                                     // initialise data in time
    y_exp = entry(:,2);
    
    ntot = size(t_exp,1);
    
    t_exp_r = t_exp(1:(ntot-1),:);
    
    delta_t = t_exp(2:ntot,1)-t_exp(1:(ntot-1),1);
    delta_y = y_exp(1:(ntot-1),1)-y_exp(2:ntot,1);                       // inverted to keep a positive difference (dec reasing function)
    
    tol = er_tol*r_tol;                                                  // tolerance for the selection of points
    
// beginning of the computation --> change of directory
    
    chdir(T_results);
    
// algorithm for selection of the plateaus
    
    i = 1;
    j = 1;
    
    n_stop = 5;
    
    while i < ntot-n_stop
    
        while abs(delta_y(i)) < tol                                                                // select the last ^point before bubling
           i = i+1;
           if i >= ntot-n_stop then break; end;                                                    // security to ensutre that i < itot
        end
    
        a1(j) = i-1;
    
        i = i+1;
    
        if i >= ntot-n_stop then break; end;                                                      // security to ensutre that i < itot
    
        while abs(delta_y(i)) > tol | abs(delta_y(i+1)) > tol | abs(delta_y(i+2)) > tol           // select the first point of the next good zone that fulfills the tolerance
            i = i+1;
            if i >= ntot-n_stop then break; end;                                                  // security to ensutre that i < itot
        end
    
        a2(j) = i;
    
        j = j+1;
    
    end
    
////  Filtering the too small plateaus

    ncell = min([size(a1,1) size(a2,1)]);                                                       // number of parts of the curve

    for i = 1:(ncell-1)
        length_cell(i) = a1(i+1)-a2(i)+1;                                                       // length of the parts
    end
    
    m_lgth = floor(mean(length_cell));                                                          // average length
    
    for i = 1:(ncell-1)
        if length_cell(i) <= floor(m_lgth/r_lgth)
            then a1(i+1) = 0; end;                                                              // selection of parts that have a too small length --> middle of bubling
    end
    
    for i = 1:(ncell-1)
        if length_cell(i) <= floor(m_lgth/r_lgth)
            then a2(i) = 0;end;                                                                 // selection of parts that have a too small length --> middle of bubling
    end
    
    na1 = a1(find(a1<>0)');                                                                      // unselection of these points
    na2 = a2(find(a2<>0)');
    
////  Definition of the plateaus characteristics (upper boundary and values)
    
    nntot = (size(na2,1)-1);                                                                    // to avoid trouble at the end
    
    tcell = cell(nntot+1,1);
    ycell = cell(nntot+1,1);
    
    tcell{1} = t_exp(1:na1(1));                                                                  // selection of the parts
    ycell{1} = y_exp(1:na1(1));
    
    for i = 1:nntot
        tcell{i+1} = t_exp(na2(i):na1(i+1),:);
        ycell{i+1} = y_exp(na2(i):na1(i+1),:);
    end
    
    t_exp_rr(1) = t_exp(na1(1));                                                                  // determination of the selected times and computation of staircase values
    y_exp_rr(1) = median(y_exp(1:na1(1)));
    
    for i = 1:nntot
        t_exp_rr(i+1) = max(tcell{i+1});
        y_exp_rr(i+1) = median(ycell{i+1});
    end
    
// Filtering to avoid increase in values instead of decrease
    
    delta_y_exp_rr = y_exp_rr(2:nntot+1)-y_exp_rr(1:nntot);                                        // difference y(i+1) - y(i)
    
    [zna_ znb_] = find(delta_y_exp_rr < 0);                                                        // selection of the data with y(i+1) > y(i) - strictly decerasing average
    
     zna = [1;zna_'+1];                                                                            // note i = 1 address the case of y(2)-y(1) --> correspond to the second object
    
    t_exp_rrz = t_exp_rr(zna);
    y_exp_rrz = y_exp_rr(zna);
    
    t_select = t_exp_rrz;                                                 // to avoid the last point
    y_select = y_exp_rrz;
    
// Computation of design of the continous function (construction of the plateaus)
    
    for i = 1:size(zna,1)                                                                           // select the corresponding parts that verify  y(i+1) > y(i)
        t_exp_rrc{i} = tcell{zna(i)};
        y_exp_rrc{i} = y_exp_rr(zna(i))*ones(tcell{zna(i)});
    end
    
    t_exp_rc = [];
    y_exp_rc = [];
    
    for i = 1:size(zna,1)                                                                           // combine the different parts for all the parts
        t_exp_rc = [t_exp_rc; t_exp_rrc{i}];
        y_exp_rc = [y_exp_rc; y_exp_rrc{i}];
    end
    
    t_staircase = t_exp_rc;
    y_staircase = y_exp_rc;
    
// assessment of the difference in length and values of the plateaus
    
    n_select = size(t_select,1);
    
    delta_t_select = t_select(2:n_select)-t_select(1:(n_select-1));                                    // computes the differences between the final plateau lengths and values
    delta_y_select = y_select(2:n_select)-y_select(1:(n_select-1));
    
    
/////////////////// savings ////////////////////////////////////////////////

    data_selected = [t_select y_select];
    
    names_result = "SciFile_txt_"+name+".txt";                                                      // name of the file containing the data
    names_result_sod = "SciFile_sod_"+name+".sod";                                                  // format of data file for scuilab = .sod
    
    mdelete(names_result);                                                                          // delete the existing file
    write(names_result,data_selected);                                                              // write the new file
    
    mdelete(names_result_sod);                                                                          // delete the existing file
    save(names_result_sod);
    
    chdir(T_wd);
    
/////////////////// Graphs ////////////////////////////////////////////////////////
    
////  Selection procedure

    // Plot of the selected and experimental data
    
    clf(i_fig);
    scf(i_fig);
    
    i_fig = i_fig+1;

    plot(t_exp,y_exp,"blue +","Markersize",7);
    plot(t_select,y_select,"red o","Markersize",7);
    plot(t_staircase,y_staircase,"red -","Markersize",7);

    xlabel("$t\ [s]$","fontsize",xfs_axis);
    ylabel("$y\ [V]$","fontsize",xfs_axis);
    leg = legend("Exp. data","Selected data","Staircase function",1);
    leg.marks_count = 1;
    leg.line_width = 0.05;
    leg.line_mode = "off";
    leg.font_size = xfs_leg;
   
    title("Selection - "+ name,"fontsize",xfs_title);
    
    
// Additional graphs

    clf(i_fig);
    scf(i_fig);
    
    i_fig = i_fig+1;
    
    subplot(221)

    plot(linspace(1,n_select-1,n_select-1)',delta_t_select,"red o","Markersize",7);

    xlabel("$i$","fontsize",xfs_axis);
    ylabel("$\Delta t\ [s]$","fontsize",xfs_axis);
    
    title("Control panel - file :  ","fontsize",xfs_title);


    subplot(222)

    plot(linspace(1,n_select-1,n_select-1)',delta_y_select,"red o","Markersize",7);

    xlabel("$i$","fontsize",xfs_axis);
    ylabel("$\Delta y\ [V]$","fontsize",xfs_axis);
    
    title(name,"fontsize",xfs_title);

    subplot(223)
   
    plot(t_exp,y_exp,"blue +","Markersize",7);
    plot(t_select,y_select,"red o","Markersize",7);
    plot(t_staircase,y_staircase,"red -","Markersize",7);

    xlabel("$t\ [s]$","fontsize",xfs_axis);
    ylabel("$y\ [V]$","fontsize",xfs_axis);
    leg = legend("Exp. data","Selected data","Staircase function",1);
    leg.marks_count = 1;
    leg.line_width = 0.05;
    leg.line_mode = "off";
    leg.font_size = xfs_leg;

   
    subplot(224)
    
    plot(linspace(1,n_select-1,n_select-1)',abs(delta_y_select./delta_t_select),"red o","Markersize",7);

    xlabel("$i$","fontsize",xfs_axis);
    ylabel("$\Delta y/\Delta t\ [V/s]$","fontsize",xfs_axis);


    
